from __future__ import (absolute_import, division,
                        print_function, unicode_literals)

import logging

import os
import os.path
import subprocess
import tempfile

from . import utilfunctions


class VersionFileBuilder(object):
    VERSION_TMPL = '\n'.join([
        'const wchar_t *GITVersion = L"{version}";',
        'const wchar_t *GITDate = L"{date}";',
        ''
    ])
    notices = logging.getLogger('notices')

    def __init__(self, workfile, outfile, gitpath="git"):
        self.workfile = workfile
        self.outfile = outfile
        self.gitpath = gitpath
        self.logger = logging.getLogger(self.__class__.__name__)

    def remove_file(self, filename):
        try:
            os.remove(filename)
            self.notices.info(' Removed %s', filename)
        except OSError:
            self.logger.exception(' Unable to remove %s', filename)

    def build(self):
        self.notices.info("Building...")
        if self.need_to_update():
            self.notices.info("Generating...")

            utilfunctions.make_directory_for_filename(self.outfile)

            with open(self.outfile, "wb") as out:
                git_id = self.get_git_id()
                output = self.VERSION_TMPL.format(
                    version=git_id,
                    date=self.get_git_date())
                out.write(output.encode('utf-8'))
                with open(self.workfile, "wb") as work:
                    work.write(git_id.encode('utf-8'))

            self.notices.info(" Version information written to %s",
                              self.outfile)
        else:
            self.notices.info(" Up to date")

    def get_git_id(self):
        """Return the git id string suitable for inclusion in C code.

        Requires striping the newline from the end of the command output"""
        tf = tempfile.TemporaryFile()
        s = "describe --long --dirty --abbrev=10 --tags --always"
        s = s.split()
        s.insert(0, self.gitpath)

        subprocess.Popen(s, stdout=tf).wait()
        tf.seek(0)

        rawidoutput = tf.readline()
        idoutput = rawidoutput.decode('utf-8').split('\n')
        # split on the line ending and return the first peice which will
        # contain the entire id (all of the tags, etc) of the current
        # repository without any newlines
        return idoutput[0]

    def get_git_date(self):
        """Return the git date string suitable for includsion in C code.

        Requires stripping the newline from the end of the command output"""
        tf = tempfile.TemporaryFile()

        datecmd = 'log --pretty=format:%cd -n 1'
        datecmd = datecmd.split()
        datecmd.insert(0, self.gitpath)

        subprocess.Popen(datecmd, stdout=tf).wait()
        tf.seek(0)

        rawdateoutput = tf.readline()
        dateoutput = rawdateoutput.decode('utf-8')
        return dateoutput

    def need_to_update(self):
        """Check if we need to update the outfile
        :return: False if outfile is up to date, True otherwise
        """
        if os.path.exists(self.workfile):
            if os.path.exists(self.outfile):
                # check the id to see if it has changed
                with open(self.workfile, 'rb') as work:
                    if work.readline().decode('utf-8') == self.get_git_id():
                        return False
                    else:
                        self.notices.info(" git id changed")
            else:
                self.notices.info(" %s does not exist", self.outfile)
        else:
            self.notices.info(" %s does not exist", self.workfile)
        return True




import logging
import os
import os.path
import shutil
import tempfile

try:
    from html.parser import HTMLParser
except ImportError:
    from HTMLParser import HTMLParser

from ohm.dataclasses import Tag
from ohm.utilfunctions import update_attribute, change_filename


class OutputParser(HTMLParser):
    """Provide common functionality for the Parsers.

    Currently class passes the input file through to the passed in output"""

    def __init__(self, file):
        HTMLParser.__init__(self)

        if hasattr(file, 'write'):
            self.output_file = file
        else:
            raise ValueError("file is not a file-like")

        self.tag_stack = list()

    def handle_starttag(self, tag, attrs):
        logging.debug(" Handle starttag: %s", tag)
        logging.debug("  %s", attrs)
        self.tag_stack.append(Tag(tag, attrs))

    def handle_startendtag(self, tag, attrs):
        logging.debug(" Handle start and end tag: %s", tag)
        logging.debug("  %s", attrs)
        self.write_tag(Tag(tag, attrs))

    def handle_endtag(self, tagname):
        """Finds the matching tag recursing until finding a match"""
        logging.debug(" End tag: %s", tagname)
        tag = self.find_match(tagname)
        self.write_tag(tag)

    def handle_data(self, data):
        logging.debug(" Data: %s", data)
        if len(self.tag_stack) > 0:
            tag = self.tag_stack.pop()
            if tag.data is not None:
                tag.data += data
            else:
                tag.data = data
            self.tag_stack.append(tag)
        else:
            self.output_file.write(data)

    def find_match(self, tagtofind, indent=0):
        """Recursive function to match the tag"""
        tag = self.tag_stack.pop()
        if not tag.name == tagtofind:
            logging.warning(" %smismatched tag found (expected %s, found %s)",
                            " " * indent, tagtofind, tag.name)
            self.write_tag(tag)
            tag = self.find_match(tagtofind)

        return tag

    def write_tag(self, tag):
        """Write a tag out if it is top level.

        When tag stack is empty, writes tag to file passed in the constructor.
        When tag stack is not empty formats the tag and sets (or if the next
        tag.data is not None, appends) the formatted string."""
        if tag.data is None:
            s = "<%s%s />" % (tag.name, self.format_attributes(tag))
        else:
            s = "<%s%s>%s</%s>" % (
                tag.name, self.format_attributes(tag), tag.data, tag.name)

        if len(self.tag_stack) > 0:
            tag = self.tag_stack.pop()
            if tag.data is not None:
                tag.data += s
            else:
                tag.data = s
            self.tag_stack.append(tag)
        else:
            self.output_file.write(s)

    @staticmethod
    def format_attributes(tag):
        """Returns the attributes formatted to be placed in a tag."""
        ret = ""
        if len(tag.attributes) > 0:
            for name, value in tag.attributes:
                ret = "%s %s=\"%s\"" % (ret, name, value)
        return ret


class Stage2Parser(OutputParser):
    def __init__(self, *args, **kwargs):
        OutputParser.__init__(self, *args, **kwargs)
        self.control = []

    def handle_starttag(self, tag, attrs):
        if tag == "meta":
            if (attrs[0][0] == "name"
                    and attrs[0][1] == "control"
                    and attrs[1][0] == "content"):
                self.control.append(attrs[1][1])
        else:
            OutputParser.handle_starttag(self, tag, attrs)

    def handle_startendtag(self, tag, attrs):
        if tag == "meta":
            if (attrs[0][0] == "name"
                    and attrs[0][1] == "control"
                    and attrs[1][0] == "content"):
                self.control.append(attrs[1][1])
        else:
            OutputParser.handle_startendtag(self, tag, attrs)


class Stage3Parser(OutputParser):
    def __init__(self, options, files, extrafiles, subdir, *args, **kwargs):
        OutputParser.__init__(self, *args, **kwargs)
        self.options = options
        self.files = files
        self.extrafiles = extrafiles
        self.subdir = subdir
        logging.debug(" Subdirectory is '%s'", subdir)

    def handle_startendtag(self, tag, attrs):
        """Find the image and copy it to the stage3 folder where it should
        be in the f output."""
        alt_index = None
        srv_index = None
        if tag == "img":
            # figure out which attribute is src
            for x in range(0, len(attrs)):
                if attrs[x][0] == "src":
                    srv_index = x
                elif attrs[x][0] == "alt":
                    alt_index = x

            if attrs[srv_index][1].startswith("/"):
                # manual wants an absolute path, the help manual does not
                # support absolute path, so make sure that the image exists
                # where the absolute path indicates, then make the path into a
                # relative path with the appropriate number of updirs
                test = os.path.join(self.options.indir, attrs[srv_index][1][1:])
                if not os.path.exists(test):
                    raise IOError(
                        "Cannot find %s in base path" % (attrs[srv_index][1]))

                # try find a valid relative path
                subdirdepth = len(self.subdir.split(os.path.sep))
                prefix = "../" * subdirdepth
                relpath = os.path.join(prefix, attrs[srv_index][1][1:])
                if not os.path.exists(
                        os.path.join(self.options.indir, self.subdir, relpath)):
                    raise Exception(
                        "Cannot relativize path: %s" % (attrs[srv_index][1]))
                else:
                    attrs = update_attribute(attrs, 'src', relpath)

            location1 = os.path.join(self.options.indir, self.subdir,
                                     attrs[srv_index][1])
            location = os.path.normpath(location1)

            # check to make sure that the image I am including was in the
            # onlinehelp folder, if not change the dst name so that it will be
            # located correctly in the stage3 directory
            logging.debug("%s - %s", location, self.options.indir)
            if location.startswith(self.options.indir):
                dst1 = os.path.join(self.files['stage3'], self.subdir,
                                    attrs[srv_index][1])
                dst = os.path.normpath(dst1)
            else:
                # get extension
                basename = os.path.basename(attrs[srv_index][1])
                (name, ext) = os.path.splitext(basename)
                (f, outname) = tempfile.mkstemp(ext, name, self.files['stage3'])
                # make into a relative path
                dst1 = outname.replace(os.getcwd(), ".")

                # fix up attrs
                dst = os.path.normpath(dst1)
                filename = change_filename(dst, os.path.splitext(dst)[1],
                                           self.files['stage3'], ".",
                                           makedirs=False)
                attrs = update_attribute(attrs, 'src', filename)

            if alt_index is None:
                alt_index = srv_index
            logging.debug(" Image (%s) should be in %s and copying to %s",
                          attrs[alt_index][1], location, dst)
            try:
                if not os.path.exists(os.path.dirname(dst)):
                    os.mkdir(os.path.dirname(dst))
                shutil.copy2(location, dst)
            except:
                logging.exception(" '%s' does not exist", location)
                raise

            self.extrafiles.append(dst)
        OutputParser.handle_startendtag(self, tag, attrs)


class Stage4Parser(OutputParser):
    def __init__(self, files, options, subdir, *args, **kwargs):
        OutputParser.__init__(self, *args, **kwargs)
        self.files = files
        self.options = options
        self.subdir = subdir

    def handle_starttag(self, tag, attrs):
        if tag == "a":
            for name, value in attrs:
                attrs = self._handle_start_tag(attrs, name, value)
        OutputParser.handle_starttag(self, tag, attrs)

    def _handle_start_tag(self, attrs, name, value):
        if name == "href" and value.startswith("/"):
            if value.startswith("/"):
                # manual wants an absolute path, the help manual does not
                # support absolute path, so make sure that the image exists
                # where the absolute path indicates, then make the path into a
                # relative path with the appropriate number of up dirs

                # value is absolute but we need it relative, so drop the slash
                if not self._path_exists(value[1:]):
                    raise ValueError("Cannot find %s in base path", value)

                # try find a valid relative path
                subdirdepth = len(self.subdir.split(os.path.sep))
                prefix = "../" * subdirdepth
                relpath = os.path.join(prefix, value[1:])
                if not self._path_exists(self.subdir, relpath):
                    raise ValueError("Cannot relativize path: %s", value)
                else:
                    attrs = update_attribute(attrs, 'src', relpath)
                    value = relpath
        if name == "href" and not value.startswith("http://"):
            # make sure the file being referred to exists in stage3
            check_ref = change_filename(value, ".stage3", ".",
                                        self.files['stage3'],
                                        makedirs=False)
            if not os.path.exists(check_ref):
                logging.warning(
                    " File (%s) does not exist to be bundled into archive!",
                    check_ref)

            fixed_ref = change_filename(value, ".htm", ".", ".",
                                        makedirs=False)
            attrs = update_attribute(attrs, "href", fixed_ref)
        return attrs

    def _path_exists(self, *paths):
        return os.path.exists(os.path.join(self.options.indir, *paths))


class Stage5Parser(OutputParser):
    def __init__(self, *args, **kwargs):
        OutputParser.__init__(self, *args, **kwargs)
        self.title = "Untitled"

    def handle_endtag(self, tag):
        if tag == "title":
            t = self.tag_stack.pop()
            self.title = t.data
            self.tag_stack.append(t)
        elif tag == "h1" and self.title == "Untitled":
            # make first h1 tag found into the title will be overwritten by
            # a title tag if it exists.
            t = self.tag_stack.pop()
            self.title = t.data
            self.tag_stack.append(t)
        OutputParser.handle_endtag(self, tag)



import logging
import os.path
import sys

import argparse

from ohm.version_file_builder import VersionFileBuilder


def main(argv):
    parser = argparse.ArgumentParser(
        description="Build .cpp using vcs tool for inclusion in application")
    parser.add_argument('job', choices=['build', 'rebuild', 'clean'])
    parser.add_argument('outfile', type=os.path.normpath,
                        help="output .cpp file")
    parser.add_argument('workfile', type=os.path.normpath,
                        help="Temp file")
    parser.add_argument("--gitpath",
                        help="name of git executable to work with."
                        "(default is 'git'",
                        default='git')

    options = parser.parse_args(argv)

    maker = VersionFileBuilder(options.workfile, options.outfile,
                               options.gitpath)

    logging.basicConfig(level=logging.INFO)

    if options.job == "build":
        maker.build()
    else:
        parser.error("Invalid JOB type. Use build, rebuild, or clean.")
        sys.exit(2)

    sys.exit(0)


if __name__ == "__main__":
    main(sys.argv[1:])



import argparse
import atexit
import logging
import os
import shutil
import sys
import tempfile

from ohm.utilfunctions import rmtree_error_handler
from ohm.jobs.build import build

try:
    import markdown
except ImportError:
    markdown = None
    print("ERROR: Unable to import markdown the markup parser.")
    print(" Make sure that markdown has been installed")
    print("  see the ReadMe.txt for more information")
    raise


def main(argv):
    parser = argparse.ArgumentParser(
        description="Build the online help database")
    parser.add_argument("-q", "--quiet", action="store_const",
                        dest="quiet", default=logging.INFO,
                        const=logging.WARNING,
                        help="don't print most status messages to stdout")
    parser.add_argument("-d", "--debug", action="store_const",
                        default=logging.INFO, const=logging.DEBUG,
                        help="print debugging information to the screen")

    type_subparsers = parser.add_subparsers(title='operation')

    bld_ps = type_subparsers.add_parser('build',
                                        help='Assemble help database')
    bld_ps.set_defaults(function=build, type="build")
    bld_ps.add_argument('outfile', type=os.path.normpath,
                        help="location to write .htb file to")
    bld_ps.add_argument('indir', type=os.path.normcase,
                        help="path that contains the info to put the OUTFILE")
    bld_ps.add_argument("-t", "--temp",
                        help="directory for intermediate build files"
                        "(Uses system temp directory by default)",
                        metavar="TEMPDIR")
    bld_ps.add_argument("-c", "--cfile", default=None, dest="carrayfilename",
                        metavar="FILE",
                        help="file to put the htb index in (c code)")
    bld_ps.add_argument("-a", "--always", action="store_true",
                        default=False, dest="always_build",
                        help="builder should always build source files")

    args = parser.parse_args(argv)

    console_format = logging.Formatter(fmt='%(levelname)7s:%(message)s')
    console = logging.StreamHandler()
    console.setLevel(args.debug)

    console.setFormatter(console_format)
    logger = logging.getLogger('')
    logger.setLevel(args.debug)
    logger.addHandler(console)

    notices_console = logging.StreamHandler()
    notices_console.setFormatter(console_format)

    notices_logger = logging.getLogger('notices')
    notices_logger.setLevel(args.quiet)
    notices_logger.addHandler(notices_console)
    notices_logger.propagate = False

    if not args.temp:
        logging.info("No working directory set. Creating one in system temp.")
        args.temp = tempfile.mkdtemp()

        def cleanup(dir):
            if os.path.exists(dir):
                shutil.rmtree(args.temp, onerror=rmtree_error_handler)

        atexit.register(cleanup, args.temp)

    if 'function' in args:
        return args.function(args)
    else:
        parser.error("No sub-command selected")


def enum_directories(dir):
    ret = os.path.dirname(dir).split(os.path.sep)
    if len(ret) == 1 and len(ret[0]) == 0:
        ret = list()
    logging.debug(" Enum directories %s", str(ret))
    return ret


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



from __future__ import absolute_import

import logging
import os
import shutil
import zipfile

import markdown

from ..help_parsers.helpparsers import (Stage2Parser, Stage3Parser,
                                        Stage4Parser, Stage5Parser)
from ..utilfunctions import change_filename


def build(args):
    """Compiles the files in args.indir to the archive output args.outfile.

    Compiled in several stages:
        - stage1: Transform all input files with markdown placing the results
        into args.temp+"/stage1".
        - stage2: Parses and strips the output of stage1 to build the list
        that will be made into the c-array that contains the compiled names for
        the detailed help of each control.
        - stage3: Parses the output of stage2 to grab the images that are
        referred to in the the output of stage1
        - stage4: Parses the output of stage1 to fix the relative hyperlinks in
        the output so that they will refer correctly to the correct files when
        in output file.
        - stage5: Generate the index and table of contents for the output file
        from the output of stage4.
        - stage6: Zip up the output of stage5 and put it in the output location.
    """
    notices = logging.getLogger('notices')
    notices.info("Building...")
    logging.debug("Using '%s' as working directory", args.temp)
    logging.debug("Using '%s' as output file", args.outfile)
    logging.debug("Using '%s' as input directory", args.indir)

    files = generate_paths(args)

    if should_build(args):
        _do_build(args, files, notices)
    else:
        notices.info(" Up to date.")


def _do_build(args, files, notices):
    help_array = list()
    extra_files = list()
    notices.info(" Processing input files:")
    for file_name in generate_file_list(args.indir, ".help"):
        notices.info("  %s", file_name)
        logging.info("   Stage 1")
        name1 = process_input_stage1(file_name, args, files)

        logging.info("   Stage 2")
        name2 = process_input_stage2(name1, args, files, help_array)

        logging.info("   Stage 3")
        name3 = process_input_stage3(name2, args, files, extra_files)

        logging.info("   Stage 4")
        name4 = process_input_stage4(name3, args, files)
    logging.info(" Stage 5")
    process_input_stage5(args, files, extra_files)
    logging.info(" Stage 6")
    process_input_stage6(args, files)
    logging.info(" Generating .cpp files")
    generate_cpp_files(args, files, help_array)
    notices.info("....Done.")


def generate_paths(options):
    """Generates the names of the paths that will be needed by the compiler.

    Generated names will be returned as a dictionary."""
    paths = dict()

    for i in range(1, 7):
        stage_name = 'stage{0}'.format(i)
        paths[stage_name] = os.path.join(options.temp, stage_name)

    for path in paths.values():
        if not os.path.exists(path):
            logging.debug(" Making %s", path)
            os.makedirs(path)

    return paths


def should_build(options):
    """Should we build the output file?
    :param options arguments from command line
    :return True if the output file should be built/rebuilt
    """
    logger = logging.getLogger('should_build')
    if options.always_build:
        logger.info(" Always building")
        return True
    elif not os.path.exists(options.outfile):
        logger.info(" Outfile does not exist")
        return True
    elif options.carrayfilename and not os.path.exists(options.carrayfilename):
        logger.info(" .cpp file does not exist")
        return True
    elif check_source_newer_than_outfile(options):
        logger.info(" Source files are newer than output")
        return True
    return False


def check_source_newer_than_outfile(options):
    try:
        outfile_time = os.path.getmtime(options.outfile)

        for d_path, d_names, f_names in os.walk(options.indir):
            for f_name in f_names:
                f_path = os.path.join(d_path, f_name)
                if os.path.getmtime(f_path) > outfile_time:
                    logging.info(
                        "%s has been changed since outfile. Causing build",
                        f_path)
                    return True
                elif os.path.getctime(f_path) > outfile_time:
                    logging.info(
                        "%s has been created since outfile. Causing build",
                        f_path)
                    return True

    except OSError as e:
        logging.exception("OS does not like %d. Forcing build.", e.filename)
        return True
    return False


def generate_file_list(directory, extension):
    for path, dirs, files in os.walk(directory):
        for file_name in files:
            if os.path.splitext(file_name)[1] == extension:
                yield os.path.join(path, file_name)


def process_input_stage1(file, options, files):
    infile = open(file, mode="r")
    input = infile.read()
    infile.close()

    md = markdown.Markdown(
        extensions=['markdown.extensions.toc']
    )
    output = md.convert(input)

    outfile_name = change_filename(file, ".stage1", options.indir, files['stage1'])

    outfile = open(outfile_name, mode="w")
    outfile.write(output)
    outfile.close()
    return outfile_name


def process_input_stage2(file, options, files, helparray):
    infile = open(file, mode="r")
    input = infile.read()
    infile.close()

    outname = change_filename(file, ".stage2", files['stage1'], files['stage2'])
    outfile = open(outname, mode="w")

    parser = Stage2Parser(file=outfile)
    parser.feed(input)
    parser.close()
    outfile.close()

    if len(parser.control) > 0:
        filename_in_archive = change_filename(outname, ".htm", files['stage2'], ".", makedirs=False)
        for control in parser.control:
            helparray.append((control, filename_in_archive))
            logging.debug(" Control name %s", control)

    return outname


def process_input_stage3(file, options, files, extrafiles):
    infile = open(file, mode="r")
    input = infile.read()
    infile.close()

    outname = change_filename(file, ".stage3", files['stage2'], files['stage3'])
    outfile = open(outname, mode="w")

    # figure out what subdirectory of the onlinehelp I am in
    subdir = os.path.dirname(outname).replace(os.path.normpath(files['stage3']), "")
    if subdir.startswith(os.path.sep):
        subdir = subdir.replace(os.path.sep, "", 1)  # I only want to remove the leading sep

    parser = Stage3Parser(options, files, file=outfile, extrafiles=extrafiles, subdir=subdir)
    parser.feed(input)
    parser.close()
    outfile.close()

    return outname


def process_input_stage4(file, options, files):
    """Fix the a tags so that they point to the htm files that will be in the output archive."""
    infile = open(file, mode="r")
    input = infile.read()
    infile.close()

    outname = change_filename(file, ".stage4", files['stage3'], files['stage4'])
    outfile = open(outname, mode="w")

    # figure out what subdirectory of the onlinehelp I am in
    subdir = os.path.dirname(outname).replace(os.path.normpath(files['stage4']), "")
    if subdir.startswith(os.path.sep):
        subdir = subdir.replace(os.path.sep, "", 1)  # I only want to remove the leading sep

    parser = Stage4Parser(files=files, file=outfile, options=options,
                          subdir=subdir)
    parser.feed(input)
    parser.close()
    outfile.close()

    return outname

HTB_HEADER_TEMPLATE = "\n".join([
    "Contents file={contents}",
    "Index file={index}",
    "Title={title}",
    "Default topic={default}",
    "Charset=UTF-8",
    ""])

INDEXES_FILE_TEMPLATE = "\n".join([
    '%(tab)s<li>',
    '%(tab)s\t<object type="text/sitemap">',
    '%(tab)s\t\t<param name="Name" value="%(name)s">',
    '%(tab)s\t\t<param name="Local" value="%(file)s">',
    '%(tab)s\t</object>',
    ''])


def process_input_stage5(options, files, extrafiles):
    """Generate the index and table of contents"""

    # write header file
    header_file_name = os.path.join(files['stage5'], "header.hhp")
    with open(header_file_name, mode="w") as header_file:
        header_file.write(HTB_HEADER_TEMPLATE.format(
            contents="contents.hhc",
            index="index.hhk",
            title="wxLauncher User Guide",
            default="index.htm",
        ))

    # generate both index and table of contents
    tocfile_name = os.path.join(files['stage5'], "contents.hhc")
    tocfile = open(tocfile_name, mode="w")
    tocfile.write("<ul>\n")
    toclevel = 1

    indexfile_name = os.path.join(files['stage5'], "index.hhk")
    indexfile = open(indexfile_name, mode="w")
    indexfile.write("<ul>\n")

    help_files = generate_file_list(files['stage4'], ".stage4")

    last_path_list = []
    for path, dirs, thefiles in os.walk(files['stage4']):
        """new directory. If the directory is not the base directory then check if there will be an index.htm for this directory.
    If there is an index.htm then parse it for the title so that the subsection will have the title of the index.htm as the name of the subsection.
    If there is no index.htm then make a default one and use the name of the directory as the subsection name.

    Note that this algorithm requires that os.walk generates the names in alphabetical order."""
        # relativize directory path for being in the archive
        path_list = find_path_in_archive(path, files['stage4'])
        logging.debug("Processing directory '%s'", os.path.sep.join(path_list))

        if len(path_list) == 1 and path_list[0] == '':
            path_list = []
        level = len(path_list)

        # parse the index.help to get the name of the section
        index_file_name = os.path.join(path, "index.stage4")
        # relativize filename for being in the archive
        index_in_archive = change_filename(index_file_name, ".htm", files['stage4'], ".", False)
        index_in_archive = index_in_archive.replace(os.path.sep,
                                                    "/")  # make the separators the same so that it doesn't matter what platform the launcher is built on.
        # find the title
        outindex_name = change_filename(index_file_name, ".htm", files['stage4'], files['stage5'])
        outindex = open(outindex_name, mode="w")

        inindex = open(index_file_name, mode="r")
        input = inindex.read()
        inindex.close()

        parser = Stage5Parser(file=outindex)
        parser.feed(input)
        parser.close()
        outindex.close()

        tocfile.write(
            generate_sections(path_list, last_path_list, index_filename=index_in_archive, section_title=parser.title))
        last_path_list = path_list

        if level > 0:
            # remove index.htm from thefiles because they are used by the section names
            try:
                thefiles.remove('index.stage4')
            except ValueError:
                logging.warning("Directory %s does not have an index.help",
                                os.path.sep.join(path_list))

        for file in thefiles:
            full_filename = os.path.join(path, file)
            # relativize filename for being in the archive
            filename_in_archive = change_filename(full_filename, ".htm", files['stage4'], ".", False)
            # find the title
            outfile_name = change_filename(full_filename, ".htm", files['stage4'], files['stage5'])
            outfile = open(outfile_name, mode="w")

            infile = open(full_filename, mode="r")
            input = infile.read()
            infile.close()

            parser = Stage5Parser(file=outfile)
            parser.feed(input)
            parser.close()
            outfile.close()

            filename_in_indexes = filename_in_archive.replace('\\', '/')

            tocfile.write(INDEXES_FILE_TEMPLATE % {
                "tab": "\t"*level,
                "name": parser.title,
                "file": filename_in_indexes
            })

            indexfile.write(INDEXES_FILE_TEMPLATE % {
                "tab": "\t",
                "name": parser.title,
                "file": filename_in_indexes
            })

    tocfile.write(generate_sections([], last_path_list))
    tocfile.write("</ul>\n")
    tocfile.close()
    indexfile.close()

    # copy the extra files (i.e. images) from stage3
    for extrafilename in extrafiles:
        logging.debug(" Copying: %s", extrafilename)
        dst = change_filename(extrafilename, None, orginaldir=files['stage3'], destdir=files['stage5'])
        shutil.copy2(extrafilename, dst)


def generate_sections(path_list, last_path_list, basetab=0, orginal_path_list=None, index_filename=None,
                      section_title=None):
    """Return the string that will allow me to write in the correct section."""
    logging.debug("   generate_sections(%s, %s, %d, %s)", str(path_list), str(last_path_list), basetab,
                  orginal_path_list)

    if orginal_path_list == None:
        orginal_path_list = path_list

    if len(path_list) > 0 and len(last_path_list) > 0 and path_list[0] == last_path_list[0]:
        logging.debug("    matches don't need to do anything")
        return generate_sections(path_list[1:], last_path_list[1:], basetab + 1, orginal_path_list, index_filename,
                                 section_title)

    elif len(path_list) > 0 and len(last_path_list) > 0:
        logging.debug("    go down then up")
        s = generate_sections([], last_path_list, basetab, orginal_path_list, index_filename, section_title)
        s += generate_sections(path_list, [], basetab, orginal_path_list, index_filename, section_title)
        return s

    elif len(path_list) > 0 and len(last_path_list) == 0:
        logging.debug("    go up (deeper)")
        if index_filename == None:
            raise Exception("index_filename is None")
        if section_title == None:
            title = path_list[len(path_list) - 1]
        else:
            title = section_title

        s = ""
        for path in path_list:
            s += """%(tab)s<li> <object type="text/sitemap">\n%(tab)s     <param name="Name" value="%(name)s">\n%(tab)s     <param name="Local" value="%(file)s">\n%(tab)s    </object>\n%(tab)s     <ul>\n""" % {
                "tab": "     " * basetab,
                "name": title,
                "file": index_filename}

        return s
    elif len(path_list) == 0 and len(last_path_list) > 0:
        logging.debug("    do down")
        s = ""
        basetab += len(last_path_list)
        for path in last_path_list:
            s += """%(tab)s</ul>\n""" % {'tab': "     " * (basetab)}
            basetab -= 1
        return s

    elif len(path_list) == 0 and len(last_path_list) == 0:
        logging.debug("    both lists empty, doing nothing")
        return ''

    else:
        raise Exception("Should never get here")
        return ""


def find_path_in_archive(pLeft, pRight):
    """Return the parts of the path that is in 'pLeft' but not in 'pRight'"""
    pLeft = os.path.normpath(pLeft)
    pRight = os.path.normpath(pRight)

    lLeft = pLeft.split(os.path.sep)
    lRight = pRight.split(os.path.sep)

    return find_path_in_archive_helper(lLeft, lRight)


def find_path_in_archive_helper(lLeft, lRight):
    logging.debug("make_path_in_archive_helper(%s, %s)",
                  str(lLeft), str(lRight))
    if len(lLeft) > 0 and len(lRight) > 0 and lLeft[0] == lRight[0]:
        return find_path_in_archive_helper(lLeft[1:], lRight[1:])
    elif len(lLeft) > 0 and len(lRight) == 0:
        return lLeft
    else:
        return []


def process_input_stage6(options, stage_dirs):
    # make sure that the directories exist before creating file
    outfile_path = os.path.dirname(options.outfile)
    if os.path.exists(outfile_path):
        if os.path.isdir(outfile_path):
            pass
        else:
            log.error(" %s exists but is not a directory", outfile_path)
    else:
        os.makedirs(outfile_path)

    outzip = zipfile.ZipFile(options.outfile, mode="w", compression=zipfile.ZIP_DEFLATED)

    for path, dirs, files in os.walk(stage_dirs['stage5']):
        for filename in files:
            full_filename = os.path.join(path, filename)
            arcname = change_filename(full_filename, None, stage_dirs['stage5'], ".", False)
            logging.debug(" Added %s as %s", full_filename, arcname)
            outzip.write(full_filename, arcname)

    outzip.close()


def generate_cpp_files(options, files, helparray):
    if options.carrayfilename == None:
        defaultname = os.path.join(os.path.dirname(options.outfile), "helplinks.cpp")
        logging.info("Filename for c-array has not been specified.")
        logging.info(" using default %s" % (defaultname))
        options.carrayfilename = defaultname

    logging.getLogger('notices').info("Writing to %s", options.carrayfilename)
    outfile = open(options.carrayfilename, mode="w")
    outfile.write("// Generated by scripts/onlinehelpmaker.py\n")
    outfile.write("// GENERATED FILE - DO NOT EDIT\n")
    for id, location in helparray:
        outfile.write("""{%s,_T("%s")},\n""" %
                      (id, location.replace(os.path.sep, "/")))

    outfile.close()


from __future__ import (absolute_import, division,
                        print_function, unicode_literals)

import traceback
import logging
import os
import os.path


def update_attribute(attributes, name, value):
    """Update attribute name to value
    :param attributes: list of tuples to update
    :param name: name to update
    :param value: value to update name to
    :return: attributes with updated tuple
    :rtype: tuple
    """
    ret = list()
    # find name in attributes
    for n, v in attributes:
        if n == name:
            ret.append((name, value))
        else:
            ret.append((n, v))
    return tuple(ret)


def split_filename_and_anchor(filename):
    parts = filename.split("#")
    if len(parts) > 2:
        raise Exception("Was passed filename with two (or more) # in it")
    elif len(parts) == 2:
        file = parts[0]
        anchor = parts[1]
    else:
        file = parts[0]
        anchor = None
    return tuple([file, anchor])


def change_filename(filename, newext, orginaldir, destdir, makedirs=True):
    """Twiddles URL like paths into filesystem paths
    :param filename: path to twiddle
    :param newext: new extention for filename
    :param orginaldir: original directory filename is currently in
    :param destdir: directory to prepare to receive filename
    :param makedirs: create intermediate directories in destdir
    :return: the twiddled filename
    """
    filename = os.path.normpath(filename)
    filename, anchor = split_filename_and_anchor(filename)
    if len(filename) != 0:
        orginaldir = os.path.normpath(orginaldir)
        destdir = os.path.normpath(destdir)
        logging.debug("   change_filename('%s', '%s', '%s', '%s', %s)",
                      filename, newext, orginaldir, destdir, makedirs)
        outfile_name1 = filename.replace(orginaldir, ".")  # files relative name
        logging.debug("%s", outfile_name1)
        if newext == None:
            outfile_name3 = outfile_name1
        else:
            outfile_name2 = os.path.splitext(outfile_name1)[
                0]  # file's name without ext
            logging.debug("%s", outfile_name2)
            outfile_name3 = outfile_name2 + newext
        logging.debug("%s", outfile_name3)
        outfile_name4 = os.path.join(destdir, outfile_name3)
        logging.debug("%s", outfile_name4)
        outfile_name = os.path.normpath(outfile_name4)
        logging.debug("%s", outfile_name)

        # make sure that the folder exists to output, if wanted
        if makedirs:
            outfile_path = os.path.dirname(outfile_name)
            if os.path.exists(outfile_path):
                if os.path.isdir(outfile_path):
                    pass  # do nothing because the folder already exists
                else:
                    raise Exception(
                        "%s already exists but is not a directory" % (
                        outfile_path))
            else:
                os.makedirs(outfile_path)
    else:
        outfile_name = filename

    if anchor == None:
        return outfile_name
    else:
        return "#".join([outfile_name, anchor])


def make_directory_for_filename(filename):
    """Returns False if directory for filename already exists, True if
    function was successfully able to create the directory, or raises
    and exception for the error."""
    path = os.path.dirname(filename)
    if os.path.exists(path):
        if os.path.isdir(path):
            return False
        else:
            raise ExistsButNotDirectoryError(path)
    else:
        # must be a permissions issue if we end up here
        try:
            os.makedirs(path)
            return True
        except:
            raise


class ExistsButNotDirectoryError(IOError):
    def __init__(self, path="Not Specified"):
        self.path = path

    def __str__(self):
        return "Path (%s) exists but is not a directory!"


def rmtree_error_handler(function, path, excinfo):
    if function == os.remove:
        logging.warning("  Unable to remove %s", path)
    elif function == os.rmdir:
        logging.warning("  Unable to remove directory %s", path)
    else:
        (type, value, tb) = excinfo
        logging.error("***EXCEPTION:")
        logging.error(" %s: %s", type.__name__, value)
        for filename, line, function, text in traceback.extract_tb(tb):
            logging.error("%s:%d %s", os.path.basename(filename), line, function)
            logging.error("   %s", text)
